Class: TypedParams::Configuration

Inherits:
Object
  • Object
show all
Defined in:
lib/typed_params/configuration.rb

Instance Method Summary collapse

Instance Method Details

#ignore_nil_optionalsObject

ignore_nil_optionals defines how nil optionals are handled. When enabled, optional params that are nil will be dropped given the schema does not allow_nil. Essentially, they will be treated as if they weren’t provided.



10
# File 'lib/typed_params/configuration.rb', line 10

class_attribute :ignore_nil_optionals

#key_transformObject

key_transform defines the casing for parameter keys.

One of:

- :underscore
- :camel
- :lower_camel
- :dash
- nil


38
# File 'lib/typed_params/configuration.rb', line 38

class_attribute :key_transform

#path_transformObject

path_transform defines the casing for parameter paths.

One of:

- :underscore
- :camel
- :lower_camel
- :dash
- nil


24
# File 'lib/typed_params/configuration.rb', line 24

class_attribute :path_transform